@@ -5,6 +5,7 @@ from __future__ import annotations
 from dataclasses import dataclass, field
 import logging
 
+from .id_utils import replace_complex_id_to_simple
 from ._victron_enums import DeviceType
 from .constants import MetricKind, MetricNature, MetricType, RangeType, ValueType, VictronEnum
 
@@ -16,6 +17,10 @@ def topic_to_device_type(topic_parts: list[str]) -> DeviceType | None:
     if len(topic_parts) < 2:
         return None
     native_device_type = topic_parts[2]
+    # For whatever reason some system  send topics to  N/+/platform/0/ProductName
+    if native_device_type == "platform":
+        native_device_type = "system"
+    # for settings like N/+/settings/0/Settings/CGwacs/AcPowerSetPoint
     if native_device_type == "settings":
         native_device_type = topic_parts[5]
     result = DeviceType.from_code(native_device_type)
@@ -33,15 +38,17 @@ class TopicDescriptor:
     unit_of_measurement: str | None = None
     metric_type: MetricType = MetricType.NONE
     metric_nature: MetricNature = MetricNature.NONE
-    device_type: DeviceType | None = None
     value_type: ValueType | None = None
     precision: int | None = 2
     enum: type[VictronEnum] | None = None
     min_max_range: RangeType = RangeType.STATIC
-    min: int | None = None
-    max: int | None = None
+    min: float | int | None = None
+    max: float | int | None = None
+    step: float | int | None = None
     is_adjustable_suffix: str | None = None
     key_values: dict[str, str] = field(default_factory=dict)
+    experimental: bool = False
+    generic_name: str | None = None
 
     def __repr__(self) -> str:
         """Return a string representation of the topic."""
@@ -50,26 +57,26 @@ class TopicDescriptor:
             f"message_type={self.message_type}, "
             f"short_id={self.short_id}, "
             f"name={self.name}, "
+            f"generic_name={self.generic_name}, "
             f"unit_of_measurement={self.unit_of_measurement}, "
             f"metric_type={self.metric_type}, "
             f"metric_nature={self.metric_nature}, "
-            f"device_type={self.device_type}, "
             f"precision={self.value_type}, "
             f"precision={self.precision}, "
             f"min={self.min}, "
             f"max={self.max}, "
+            f"step={self.step}, "
             f"enum={self.enum}, "
             f"is_adjustable_suffix={self.is_adjustable_suffix}, "
-            f"key_values={self.key_values})"
+            f"key_values={self.key_values}, "
+            f"experimental={self.experimental})"
         )
     
     def __post_init__(self):
         assert self.message_type == MetricKind.ATTRIBUTE or self.name is not None
         if self.value_type != ValueType.FLOAT:
             self.precision = None
-        if self.message_type != MetricKind.ATTRIBUTE and self.device_type is None:
-            self.device_type = topic_to_device_type(self.topic.split("/"))
-            assert isinstance(self.device_type, DeviceType), f"Invalid device type for topic: {self.topic}"
+        self.generic_name = replace_complex_id_to_simple(self.name) if self.name else None
         # Voltage default
         if self.metric_type == MetricType.VOLTAGE:
             if self.unit_of_measurement is None:
@@ -215,26 +222,18 @@ class ParsedTopic:
         wildcard_topic_parts = topic_parts.copy()
 
         installation_id = topic_parts[1]
-        wildcard_topic_parts[1] = "+"
-        native_device_type = topic_parts[2]
-        # For whatever reason some system  send topics to  N/+/platform/0/ProductName
-        if native_device_type == "platform":
-            native_device_type = "system"
-        # for settings like N/ce3f0ae5476a/settings/0/Settings/CGwacs/AcPowerSetPoint
-        if native_device_type == "settings":
-            native_device_type = topic_parts[5]
-
+        wildcard_topic_parts[1] = "##installation_id##"
         device_type = topic_to_device_type(topic_parts)
         if device_type is None: # This can happen as we register for the attribute topics
-            _LOGGER.warning("Unknown device type: %s, topic: %s", native_device_type, topic)
+            _LOGGER.warning("Unknown device type for topic: %s", topic)
             # If the device type is unknown, we cannot create a ParsedTopic
             return None
 
         device_id = topic_parts[3]
-        wildcard_topic_parts[3] = "+"
+        wildcard_topic_parts[3] = "##device_id##"
 
         wildcards_with_device_type = ParsedTopic.normalize_topic("/".join(wildcard_topic_parts))
-        wildcard_topic_parts[2] = "+"
+        wildcard_topic_parts[2] = "##device_type##"
         wildcards_without_device_type = ParsedTopic.normalize_topic("/".join(wildcard_topic_parts))
 
         return cls(
@@ -250,13 +249,13 @@ class ParsedTopic:
         self._key_values = self.get_key_values(topic_desc)
         self._key_values.update(topic_desc.key_values)
         self._short_id = self._replace_ids(topic_desc.short_id)
-        assert topic_desc.name is not None
+        assert topic_desc.name is not None, f"TopicDescriptor name is None for topic: {topic_desc.topic}"
         self._name = self._replace_ids(topic_desc.name)
 
     def match_from_list(self, topic_desc_list: list[TopicDescriptor]) -> TopicDescriptor |None:
         topic_parts = self.full_topic.split("/")
-        topic_parts[1] = "+"
-        topic_parts[3] = "+"
+        topic_parts[1] = "{installation_id}"
+        topic_parts[3] = "{device_id}"
         normalized_topic = "/".join(topic_parts)
         for topic_desc in topic_desc_list:
             if topic_desc.topic == normalized_topic:
@@ -267,24 +266,23 @@ class ParsedTopic:
     def short_id(self) -> str:
         return self._short_id
 
-    @property
-    def name(self) -> str:
-        assert self._name is not None
-        return self._name
-
     @property
     def key_values(self) -> dict[str, str]:
         assert self._key_values is not None
         return self._key_values
 
     def _replace_ids(self, string: str) -> str:
+        return ParsedTopic.replace_ids(string, self.key_values)
+
+    @staticmethod
+    def replace_ids(string: str, key_values: dict[str, str]) -> str:
         """Replace placeholders in the string with matched items from self.key_values."""
         import re
 
         def replace_match(match):
             key = match.group(1)
-            if key in self.key_values:
-                return self.key_values[key]
+            if key in key_values:
+                return key_values[key]
             return match.group(0)  # Leave the placeholder unchanged if no match
 
         # Match {key} in the string
