@@ -1,13 +1,14 @@
 """Module to communicate with the Venus OS MQTT Broker."""
 
 import asyncio
+import copy
 import json
 import logging
 import random
 import ssl
 import re
 import string
-from typing import Any, Callable, Optional, Tuple
+from typing import Any, Callable, Optional
 
 import paho.mqtt.client as mqtt
 from paho.mqtt.client import Client as MQTTClient, PayloadType
@@ -15,10 +16,13 @@ from paho.mqtt.enums import CallbackAPIVersion
 from paho.mqtt.reasoncodes import ReasonCode
 from paho.mqtt.properties import Properties
 
+from .writable_metric import WritableMetric
+
 from ._victron_topics import topics
-from .constants import TOPIC_INSTALLATION_ID
-from .data_classes import ParsedTopic, TopicDescriptor
-from .device import Device
+from ._victron_enums import DeviceType
+from .constants import TOPIC_INSTALLATION_ID, MetricKind, OperationMode
+from .data_classes import ParsedTopic, TopicDescriptor, topic_to_device_type
+from .device import Device, FallbackPlaceholder, MetricPlaceholder
 from .metric import Metric
 
 _LOGGER = logging.getLogger(__name__)
@@ -70,7 +74,9 @@ class Hub:
         model_name: str | None = None,
         serial: str | None = "noserial",
         topic_prefix: str | None = None,
-        topic_log_info: str | None = None
+        topic_log_info: str | None = None,
+        operation_mode: OperationMode = OperationMode.FULL,
+        device_type_exclude_filter: list[DeviceType] | None = None,
     ) -> None:
         """Initialize."""
         global running_client_id
@@ -102,9 +108,19 @@ class Hub:
             raise TypeError(f"serial must be a string or None, got type={type(serial).__name__}, value={serial!r}")
         if topic_prefix is not None and not isinstance(topic_prefix, str):
             raise TypeError(f"topic_prefix must be a string or None, got type={type(topic_prefix).__name__}, value={topic_prefix!r}")
+        if topic_log_info is not None and not isinstance(topic_log_info, str):
+            raise TypeError(f"topic_log_info must be a string or None, got type={type(topic_log_info).__name__}, value={topic_log_info!r}")
+        if not isinstance(operation_mode, OperationMode):
+            raise TypeError(f"operation_mode must be an instance of OperationMode, got type={type(operation_mode).__name__}, value={operation_mode!r}")
+        if device_type_exclude_filter is not None and not isinstance(device_type_exclude_filter, list):
+            raise TypeError(f"device_type_exclude_filter must be a list or None, got type={type(device_type_exclude_filter).__name__}, value={device_type_exclude_filter!r}")
+        if device_type_exclude_filter is not None:
+            for device_type in device_type_exclude_filter:
+                if not isinstance(device_type, DeviceType):
+                    raise TypeError(f"device_type_filter must contain only DeviceType instances, got type={type(device_type).__name__}, value={device_type!r}")
         _LOGGER.info(
-            "Initializing Hub(host=%s, port=%d, username=%s, use_ssl=%s, installation_id=%s, model_name=%s, topic_prefix=%s)",
-            host, port, username, use_ssl, installation_id, model_name, topic_prefix
+            "Initializing Hub[ID: %d](host=%s, port=%d, username=%s, use_ssl=%s, installation_id=%s, model_name=%s, topic_prefix=%s, operation_mode=%s, device_type_exclude_filter=%s)",
+            self._instance_id, host, port, username, use_ssl, installation_id, model_name, topic_prefix, operation_mode, device_type_exclude_filter
         )
         self._model_name = model_name
         self.host = host
@@ -125,18 +141,56 @@ class Hub:
         self._connected_failed_attempts = 0
         self._on_new_metric: CallbackOnNewMetric | None = None
         self._topic_log_info = topic_log_info
+        self._operation_mode = operation_mode
+        self._device_type_exclude_filter = device_type_exclude_filter
         # The client ID is generated using a random string and the instance ID. It has to be unique between all clients connected to the same mqtt server. If not, they may reset each other connection.
         random_string = ''.join(random.choices(string.ascii_letters + string.digits, k=8))
         self._client_id = f"victron_mqtt-{random_string}-{self._instance_id}"
         self._keepalive_counter = 0
-        self._new_metrics: list[Tuple[Device, Metric]] = []
+        self._metrics_placeholders: list[MetricPlaceholder] = []
+        self._fallback_placeholders: list[FallbackPlaceholder] = []
+        self._all_metrics: dict[str, Metric] = {}
+        self._first_connect = True
+
+        # Filter the active topics
+        metrics_active_topics: list[TopicDescriptor] = []
+        self._service_active_topics: dict[str, TopicDescriptor] = {}
+        for topic in topics:
+            if operation_mode != OperationMode.EXPERIMENTAL and topic.experimental:
+                continue
+            if topic.message_type == MetricKind.SERVICE:
+                self._service_active_topics[topic.short_id] = topic
+            else:        
+                #if operation_mode is READ_ONLY we should change all TopicDescriptor to SENSOR and BINARY_SENSOR
+                if operation_mode != OperationMode.READ_ONLY or topic.message_type in [MetricKind.ATTRIBUTE, MetricKind.SENSOR, MetricKind.BINARY_SENSOR]:
+                    metrics_active_topics.append(topic)
+                else: # READ ONLY and writable topic
+                    #deep copy the topic
+                    new_topic = copy.deepcopy(topic)
+                    new_topic.message_type = MetricKind.BINARY_SENSOR if topic.message_type == MetricKind.SWITCH else MetricKind.SENSOR
+                    metrics_active_topics.append(new_topic)
 
         # Replace all {placeholder} patterns with + for MQTT wildcards
-        expanded_topics = Hub.expand_topic_list(topics)
+        expanded_topics = Hub.expand_topic_list(metrics_active_topics)
+        # Apply device type filtering if specified
+        if self._device_type_exclude_filter is not None and len(self._device_type_exclude_filter) > 0:
+            relevant_topics = []
+            for td in expanded_topics:
+                if td.message_type == MetricKind.ATTRIBUTE:
+                    relevant_topics.append(td)
+                    continue
+                topic_device_types = topic_to_device_type(td.topic.split("/"))
+                assert topic_device_types is not None
+                if topic_device_types in self._device_type_exclude_filter:
+                    _LOGGER.info("Topic %s is filtered by device type: %s", td.topic, topic_device_types)
+                else:
+                    relevant_topics.append(td)
+            expanded_topics = relevant_topics
+
         def merge_is_adjustable_suffix(desc: TopicDescriptor) -> str:
             """Merge the topic with its adjustable suffix."""
             assert desc.is_adjustable_suffix is not None
-            return Hub._remove_placeholders_map(desc.topic.rsplit('/', 1)[0] + '/' + desc.is_adjustable_suffix)
+            return desc.topic.rsplit('/', 1)[0] + '/' + desc.is_adjustable_suffix
 
         # Helper to build a map where duplicate keys accumulate values in a list
         def build_multi_map(items, key_func):
@@ -200,12 +254,34 @@ class Hub:
             if self._installation_id is None:
                 _LOGGER.info("No installation ID provided, attempting to read from device")
                 self._installation_id = await self._read_installation_id()
+            # First we need to replace the installation ID in the subscription topics
+            new_list: list[str] = []
+            for topic in self._subscription_list:
+                new_topic = topic.replace("{installation_id}", self._installation_id)
+                new_list.append(new_topic)
+            self._subscription_list = new_list
+            # First setup subscriptions will happen here as we need the installation ID.
+            # Later we will do it from the connect callback
+            self._setup_subscriptions()
             self._start_keep_alive_loop()
             _LOGGER.info("Connected. Installation ID: %s", self._installation_id)
         except Exception as exc:
             _LOGGER.error("Failed to connect to MQTT broker: %s", exc, exc_info=True)
             raise CannotConnectError(f"Failed to connect to MQTT broker: {exc}") from exc
 
+    def publish(self, topic_short_id: str, device_id: str, value: str | float | int | None) -> None:
+        """Publish a message to the MQTT broker."""
+        _LOGGER.info("Publishing message to topic_short_id: %s, device_id: %s, value: %s", topic_short_id, device_id, value)
+        topic_desc = self._service_active_topics.get(topic_short_id)
+        if topic_desc is None:
+            _LOGGER.error("No active topic found for topic_short_id: %s", topic_short_id)
+            raise TopicNotFoundError(f"No active topic found for topic_short_id: {topic_short_id}")
+        assert self._installation_id is not None, "Installation ID must be set before publishing"
+        assert device_id is not None, "Device ID must be provided"
+        topic = topic_desc.topic.replace("{installation_id}", self._installation_id).replace("{device_id}", device_id)
+        payload = WritableMetric._wrap_payload(topic_desc, value) if value is not None else ""
+        self._publish(topic, payload)
+
     def _on_log(self, client: MQTTClient, userdata: Any, level:int, buf:str) -> None:
         _LOGGER.log(level, buf)
 
@@ -223,7 +299,8 @@ class Hub:
         if rc == 0:
             _LOGGER.info("Connected to MQTT broker successfully")
             self._setup_subscriptions()
-            self._loop.call_soon_threadsafe(self._connected_event.set)
+            if self._loop.is_running():
+                self._loop.call_soon_threadsafe(self._connected_event.set)
         else:
             _LOGGER.error("Failed to connect with error code: %s. flags: %s", rc, flags)
 
@@ -260,17 +337,25 @@ class Hub:
             # Check if it matches our client ID so we got full cycle or refresh
             if echo and echo.startswith(self._client_id):
                 log_debug("Full publish completed: %s", echo)
+                new_metrics: list[tuple[Device, Metric]] = []
+                for metric_placeholder in self._metrics_placeholders:
+                    metric = metric_placeholder.device.add_placeholder(metric_placeholder, self._all_metrics, self._fallback_placeholders, self)
+                    self._all_metrics[metric.short_id] = metric
+                    new_metrics.append((metric_placeholder.device, metric))
                 # We are sending the new metrics now as we can be sure that the metric handled all the attribute topics and now ready.
-                for device, new_metric in self._new_metrics:
+                for device, metric in new_metrics:
+                    metric.phase2_init(self._all_metrics)
                     try:
                         if callable(self._on_new_metric):
                             if self._loop.is_running():
                                 # If the event loop is running, schedule the callback
-                                self._loop.call_soon_threadsafe(self._on_new_metric, self, device, new_metric)
+                                self._loop.call_soon_threadsafe(self._on_new_metric, self, device, metric)
                     except Exception as exc:
                         _LOGGER.error("Error calling _on_new_metric callback %s", exc, exc_info=True)
-                self._new_metrics.clear()
-                self._loop.call_soon_threadsafe(self._first_refresh_event.set)
+                self._metrics_placeholders.clear()
+                self._fallback_placeholders.clear()
+                if self._loop.is_running():
+                    self._loop.call_soon_threadsafe(self._first_refresh_event.set)
                 return
             else:
                 log_debug("Not our echo: %s", echo)
@@ -289,7 +374,8 @@ class Hub:
 
         self._installation_id = parsed_topic.installation_id
         log_debug("Installation ID received: %s. Original topic: %s", self._installation_id, topic)
-        self._loop.call_soon_threadsafe(self._installation_id_event.set)
+        if self._loop.is_running():
+            self._loop.call_soon_threadsafe(self._installation_id_event.set)
 
     def _handle_normal_message(self, topic: str, payload: str, log_debug) -> None:
         """Handle regular MQTT message."""
@@ -320,9 +406,11 @@ class Hub:
                 return
 
         device = self._get_or_create_device(parsed_topic, desc)
-        tuple = device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload, self._loop, self, log_debug)
-        if tuple:
-            self._new_metrics.append(tuple)
+        metric_place_holder = device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload, self._loop, log_debug)
+        if isinstance(metric_place_holder, MetricPlaceholder):
+            self._metrics_placeholders.append(metric_place_holder)
+        elif isinstance(metric_place_holder, FallbackPlaceholder):
+            self._fallback_placeholders.append(metric_place_holder)
 
     async def disconnect(self) -> None:
         """Disconnect from the hub."""
@@ -335,6 +423,8 @@ class Hub:
         if self._client.is_connected():
             self._client.disconnect()
             _LOGGER.info("Disconnected from MQTT broker")
+            # Give a small delay to allow any pending MQTT messages to be processed
+            await asyncio.sleep(0.1)
         self._client = None
 
     async def _keepalive(self) -> None:
@@ -352,11 +442,12 @@ class Hub:
         _LOGGER.debug("Sending keepalive message to topic: %s", keep_alive_topic)
         self._keepalive_counter += 1
         keepalive_value = Hub.generate_keepalive_options(f"{self._client_id}-{self._keepalive_counter}")
-        self.publish(keep_alive_topic, keepalive_value)
+        self._publish(keep_alive_topic, keepalive_value)
 
-    def publish(self, topic: str, value: PayloadType) -> None:
+    def _publish(self, topic: str, value: PayloadType) -> None:
         assert self._client is not None
         prefixed_topic = self._add_topic_prefix(topic)
+        _LOGGER.debug("Publishing message to topic: %s, value: %s", prefixed_topic, value)
         self._client.publish(prefixed_topic, value)
 
     async def _keepalive_loop(self) -> None:
@@ -425,7 +516,8 @@ class Hub:
                 _LOGGER.info("Full publish completed, unsubscribing from notification")
                 if self._client is not None:
                     self._unsubscribe("#")
-                self._loop.call_soon_threadsafe(self._first_refresh_event.set)
+                if self._loop.is_running():
+                    self._loop.call_soon_threadsafe(self._first_refresh_event.set)
                 return
 
             topic_parts = topic.split("/")
@@ -449,7 +541,8 @@ class Hub:
                 self._installation_id = payload_json.get("value")
                 _LOGGER.info("Installation ID received: %s", self._installation_id)
                 if self._installation_id_event is not None:
-                    self._loop.call_soon_threadsafe(self._installation_id_event.set)
+                    if self._loop.is_running():
+                        self._loop.call_soon_threadsafe(self._installation_id_event.set)
         except Exception as exc:
             _LOGGER.error("Error processing installation ID message: %s", exc)
 
@@ -476,13 +569,19 @@ class Hub:
 
     @staticmethod
     def _remove_placeholders(topic: str) -> str:
-        return re.sub(r'\{[^}]+\}', '+', topic)
+        return re.sub(r'\{(?!installation_id\})[^}]+\}', '+', topic)
 
     @staticmethod
     def _remove_placeholders_map(topic: str) -> str:
         topic_parts = topic.split("/")
         for i, part in enumerate(topic_parts):
-            if part == "{phase}":
+            if i == 1:
+                topic_parts[i] = "##installation_id##"
+            elif i == 2 and part.startswith("{") and part.endswith("}"):
+                topic_parts[i] = "##device_type##"
+            elif i == 3:
+                topic_parts[i] = "##device_id##"
+            elif part == "{phase}":
                 topic_parts[i] = "##phase##"
             elif part.isdigit() or (part.startswith("{") and part.endswith("}")):
                 topic_parts[i] = "##num##"
@@ -507,7 +606,11 @@ class Hub:
         assert self._client is not None
         prefixed_topic = self._add_topic_prefix(topic)
         _LOGGER.debug("Subscribing to: %s", prefixed_topic)
-        self._client.subscribe(prefixed_topic)
+        try:
+            self._client.subscribe(prefixed_topic)
+        except Exception as e:
+            _LOGGER.error("Failed to subscribe to %s: %s", prefixed_topic, e)
+            raise
 
     def _unsubscribe(self, topic: str) -> None:
         """Unsubscribe from a topic with automatic prefix handling."""
@@ -518,6 +621,10 @@ class Hub:
 
     def _setup_subscriptions(self) -> None:
         """Subscribe to list of topics."""
+        if self._first_connect:
+            self._first_connect = False
+            _LOGGER.info("Installation ID is not set, skipping subscription setup")
+            return
         _LOGGER.info("Setting up MQTT subscriptions")
         assert self._client is not None
         if not self._client.is_connected():
@@ -525,8 +632,8 @@ class Hub:
         #topic_list = [(topic, 0) for topic in topic_map]
         for topic in self._subscription_list:
             self._subscribe(topic)
-
-        self._subscribe("N/+/full_publish_completed")
+        assert self.installation_id is not None
+        self._subscribe(f"N/{self.installation_id}/full_publish_completed")
         _LOGGER.info("Subscribed to full_publish_completed notification")
 
     async def _wait_for_connect(self) -> None:
@@ -617,7 +724,8 @@ class Hub:
         _LOGGER.warning("Connection to MQTT broker failed")
         self._connected_failed_attempts += 1
         if self._connected_failed_attempts >= CONNECT_MAX_FAILED_ATTEMPTS:
-            self._loop.call_soon_threadsafe(self._connected_event.set)
+            if self._loop.is_running():
+                self._loop.call_soon_threadsafe(self._connected_event.set)
 
     @staticmethod
     def expand_topic_list(topic_list: list[TopicDescriptor]) -> list[TopicDescriptor]:
@@ -681,3 +789,6 @@ class ProgrammingError(Exception):
 
 class NotConnectedError(Exception):
     """Error to indicate that we expected to be connected at this stage but is not."""
+
+class TopicNotFoundError(Exception):
+    """Error to indicate that we expected to find a topic but it is not present."""
