@@ -9,8 +9,8 @@ import asyncio
 from collections.abc import Callable
 import logging
 
-from ._victron_enums import DeviceType
-from .constants import MetricKind, MetricNature, MetricType, RangeType
+from .id_utils import replace_complex_id_to_simple, replace_complex_ids
+from .constants import MetricKind, MetricNature, MetricType
 from .data_classes import ParsedTopic, TopicDescriptor
 
 _LOGGER = logging.getLogger(__name__)
@@ -19,7 +19,7 @@ _LOGGER = logging.getLogger(__name__)
 class Metric:
     """Representation of a Victron Venus sensor."""
 
-    def __init__(self, unique_id: str, short_id: str, descriptor: TopicDescriptor, parsed_topic: ParsedTopic) -> None:
+    def __init__(self, unique_id: str, name: str, descriptor: TopicDescriptor, parsed_topic: ParsedTopic) -> None:
         """Initialize the sensor."""
         _LOGGER.debug(
             "Creating new metric: unique_id=%s, type=%s, nature=%s",
@@ -31,13 +31,15 @@ class Metric:
         self._descriptor = descriptor
         self._unique_id = unique_id
         self._value = None
-        self._short_id = short_id
-        self._name = parsed_topic.name
+        self._short_id = parsed_topic.short_id
+        self._name = name
         self._key_values: dict[str, str] = parsed_topic.key_values
         self._on_update: Callable | None = None
+        self._generic_name: str | None = None
+        self._generic_short_id: str | None = None
         _LOGGER.debug("Metric %s initialized", repr(self))
 
-    def __repr__(self) -> str:
+    def __str__(self) -> str:
         """Return the string representation of the metric."""
         key_values_str = ", ".join(f"{k}={v}" for k, v in self._key_values.items())
         key_values_part = f"key_values={{{key_values_str}}}" if key_values_str else "key_values={}"
@@ -45,15 +47,40 @@ class Metric:
             f"Metric(unique_id={self.unique_id}, "
             f"descriptor={self._descriptor}, "
             f"value={self.value}, "
-            f"generic_short_id={self.generic_short_id}, "
-            f"short_id={self.short_id}, "
-            f"name={self.name}, "
+            f"generic_short_id={self._generic_short_id}, "
+            f"short_id={self._short_id}, "
+            f"generic_name={self._generic_name}, "
+            f"name={self._name}, "
             f"{key_values_part})"
             )
 
-    def __str__(self) -> str:
-        """Return the string representation of the metric."""
-        return self.formatted_value
+    def phase2_init(self, all_metrics: dict[str, Metric]) -> None:
+        """Second phase of initializing the metric."""
+        assert self._descriptor.name is not None, f"name must be set for topic: {self._descriptor.topic}"
+        name_temp = ParsedTopic.replace_ids(self._descriptor.name, self._key_values)
+        self._name = Metric._replace_ids(name_temp, self._key_values, all_metrics)
+        self._generic_name = self._descriptor.generic_name
+        self._generic_short_id = replace_complex_id_to_simple(self._descriptor.short_id)
+
+    @staticmethod
+    def _replace_ids(orig_str: str,  key_values: dict[str, str], all_metrics: dict[str, Metric]) -> str:
+        def replace_match(match):
+            moniker = match.group('moniker')
+            key, suffix = moniker.split(':', 1)
+            assert key and suffix, f"Invalid moniker format: {moniker} in topic: {orig_str}"
+            metric = all_metrics.get(suffix)
+            if metric:
+                result = str(metric.value)
+                key_values[key] = result
+                return result
+            return key_values[key]
+
+        temp = replace_complex_ids(orig_str, replace_match)
+        if temp != orig_str:
+            _LOGGER.debug("Replaced complex placeholders in topic: %s", orig_str)
+            return temp
+
+        return ParsedTopic.replace_ids(orig_str, key_values)
 
     @property
     def formatted_value(self) -> str:
@@ -79,12 +106,20 @@ class Metric:
     @property
     def name(self) -> str:
         """Returns the short id of the metric."""
+        assert self._name is not None, f"Metric name is None for metric: {repr(self)}"
         return self._name
+    
+    @property
+    def generic_name(self) -> str:
+        """Returns the generic name of the metric."""
+        assert self._generic_name is not None, f"Metric generic_name is None for metric: {repr(self)}"
+        return self._generic_name
 
     @property
     def generic_short_id(self) -> str:
         """Returns the generic short id of the metric."""
-        return self._descriptor.short_id
+        assert self._generic_short_id is not None, f"Metric generic_short_id is None for metric: {repr(self)}"
+        return self._generic_short_id
 
     @property
     def unit_of_measurement(self) -> str | None:
@@ -111,18 +146,6 @@ class Metric:
         """Returns the precision of the metric."""
         return self._descriptor.precision
 
-    @property
-    def min_value(self) -> int | RangeType | None:
-        return self._descriptor.min
-
-    @property
-    def max_value(self) -> int | RangeType | None:
-        return self._descriptor.max
-
-    @property
-    def enum_values(self) -> list[str] | None:
-        return [e.string for e in self._descriptor.enum] if self._descriptor.enum else None
-
     @property
     def unique_id(self) -> str:
         """Return the unique id of the metric."""
@@ -143,7 +166,7 @@ class Metric:
         """Sets the on_update callback."""
         self._on_update = value
 
-    def _handle_message(self, value, event_loop: asyncio.AbstractEventLoop, log_debug: Callable[..., None]    ):
+    def _handle_message(self, value, event_loop: asyncio.AbstractEventLoop | None, log_debug: Callable[..., None]):
         """Handle a message."""
         if value != self._value:
             log_debug(
@@ -161,6 +184,9 @@ class Metric:
 
         self._value = value
 
+        if not event_loop:
+            return
+
         try:
             if callable(self._on_update):
                 if event_loop.is_running():
